(function() {
  
  var UserLibClass = function() 
  {
    
    // define javascript methods that will be called when widget need to be initialized or data updated. 
    // in these methods' context, beside jQuery, following variables are accessible:
    //   * this.elem       - parent DOM element 
    //   * User Properties - you can define user property on AdapterWidget in CPT:
    //                        1. select a AdapterWidget object 
    //                        2. right click in the property editor
    //                        3. choose "New User Property" menu item
    //                       NOTE: all user defined properties must start with '@'. 
    //                       To read a user property's value: this.readData("@UserPropertyName");
    //                       To write a user property's value: this.writeData("@UserPropertyName", value);
    //                       To invoke an action: this.invokeAction("ActionSlotPath") or this.invokeAction("ActionSlotPath", value)
    //
    //
    // [optional] if this method defined, it should return a array of required javascript and css files
    // several js libraries(jquery, underscore, backbone etc) have been loaded, so no need to specify 
    // them here again. for all loaded js libraries, refers to file CPT/grweb/public/app/graphic.php
    // below is an example for highcharts:
    this.requiredScripts = function() {   
      // *Note*, you need to put the javascript files under CPT/grweb/public/user_codes folder
      //         manually, otherwise these files cannot be deployed to device
      //return ["../user_codes/highcharts/js/highcharts.js",
      //        "../user_codes/highcharts/js/highcharts-more.js",
      //        "../user_codes/highcharts/js/exporting.js"];
    };
    
    this.setupUI = function() {
      $(this.elem).empty();
      var labelColor = this.readData("@LabelColor");
      var btn = $("<button type='button' style='height:100%; width:100%;'><span style='color:"+ labelColor + ";'>"+this.readData('@ButtonLabel')+"</span></button>").appendTo($(this.elem));
      
      btn.on('click', _.bind(this.onActionEvent, this, '@ClickAction'));
      btn.on('mousedown', _.bind(this.onActionEvent, this, '@MouseDownAction'));
      btn.on('mouseup', _.bind(this.onActionEvent, this, '@MouseUpAction'));
      btn.on('touchstart', _.bind(this.onActionEvent, this, '@TouchStartAction'));
      btn.on('touchend', _.bind(this.onActionEvent, this, '@TouchEndAction'));
      btn.on('touchcancel', _.bind(this.onActionEvent, this, '@TouchCancelAction'));
    };
    
    this.onActionEvent = function(userProp) {
      var actionPath = this.readData(userProp)
      if (actionPath.length === 0)
        console.warn(userProp+" is not configured yet");
      else
        this.invokeAction(actionPath);
    }
    
    // [required] after above required files loaded successfully, this method will be called to initialize the widget
    this.init = function() { 
      this.setupUI();
      
      $(this.elem).on('settingsChanged', _.bind(this.setupUI, this));
    };
    
    // [required] when user property changed, this callback method will be called
    // it is a good place to update widget with new data, you can read user property's value by:
    // this.readData("@UserPropertyName")
    this.update = function(userPropertyName) {
    };
    
    // [required] put possible clean up codes here
    this.cleanup = function() {
    };
                      
  };

  var userLib = new UserLibClass();
  var external_scripts = _.has(userLib, 'requiredScripts') ? userLib.requiredScripts() : [];
  if (external_scripts == null)
    external_scripts = [];
  external_scripts.push("../js/underscore.string.min.js");

  //TODO: generate settings from user_lib
  freeboard.loadWidgetPlugin({
    'external_scripts': external_scripts,
    'fill_size': false,
    'type_name': 'CPT_Button_ActionButton_Widget',
    'display_name': 'ActionButton',
    'description': 'ActionButton Widget converted from CPT Graphics'    ,
    'settings': [
      {
        'name': '@ButtonLabel',
        'display_name': '@ButtonLabel',
        'type': 'calculated',
        'default_value': '',
        'editor': '',
        'description': 'label text displayed on button'
      },
      {
        'name': '@ClickAction',
        'display_name': '@ClickAction',
        'type': 'calculated',
        'default_value': '',
        'editor': 'action',
        'description': 'action slot for widget "click" event'
      },
      {
        'name': '@LabelColor',
        'display_name': '@LabelColor',
        'type': 'text',
        'default_value': '',
        'editor': 'color'
      },
      {
        'name': '@MouseDownAction',
        'display_name': '@MouseDownAction',
        'type': 'calculated',
        'default_value': '',
        'editor': 'action',
        'description': 'action slot for widget "mousedown" event'
      },
      {
        'name': '@MouseUpAction',
        'display_name': '@MouseUpAction',
        'type': 'calculated',
        'default_value': '',
        'editor': 'action',
        'description': 'action slot for widget "mounseup" event'
      },
      {
        'name': '@TouchCancelAction',
        'display_name': '@TouchCancelAction',
        'type': 'calculated',
        'default_value': '',
        'editor': 'action',
        'description': 'action slot for widget "touchcancel" event'
      },
      {
        'name': '@TouchEndAction',
        'display_name': '@TouchEndAction',
        'type': 'calculated',
        'default_value': '',
        'editor': 'action',
        'description': 'action slot for widget "touchend" event'
      },
      {
        'name': '@TouchStartAction',
        'display_name': '@TouchStartAction',
        'type': 'calculated',
        'default_value': '',
        'editor': 'action',
        'description': 'action slot for widget "touchstart" event'
      },
      {
        'name': 'rows',
        'display_name': 'Rows',
        'type': 'option',
        'default_value': '1',
        'options': [{'name': '1 row', 'value': 1}, {'name': '2 rows', 'value': 2}, {'name': '3 rows', 'value': 3}, {'name': '4 rows', 'value': 4}, {'name': '5 rows', 'value': 5}, {'name': '6 rows', 'value': 6}, {'name': '7 rows', 'value': 7}, {'name': '8 rows', 'value': 8}]
      }      
    ], 
    newInstance: function(settings, newInstanceCallback)
    {
      var wa = new WidgetAdapter(settings);
      wa = _.extend(wa, new UserLibClass());
      newInstanceCallback(wa);
    }
  });

})();
